﻿using Steamworks;
using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Threading.Tasks;
using TMPro;
using UnityEngine;

public class DeathStrandingBackpack : Mod
{
    AssetBundle bundle;
    public static DeathStrandingBackpack instance;
    public GameObject BackpackPrefab;
    public GameObject ItemPrefab;
    public Shader shader;

    public Dictionary<string, int> items = new Dictionary<string, int>();

    public void Start()
    {
        instance = this;
        bundle = AssetBundle.LoadFromMemory(GetEmbeddedFileBytes("deathstrandingbackpack.assets"));

        BackpackPrefab = bundle.LoadAsset<GameObject>("DeathStrandingBackpack");
        ItemPrefab = bundle.LoadAsset<GameObject>("DS_BPItem");
        shader = bundle.LoadAsset<Material>("ExampleMaterial").shader;

        Debug.Log("Mod DeathStrandingBackpack has been loaded!");
        if (!RAPI.IsCurrentSceneMainMenu()) { WorldEvent_WorldLoaded(); }
        StartCoroutine(InventoryUpdater());
    }

    void Update()
    {
        if (RAPI.IsCurrentSceneMainMenu()) { return; }
        MessageHandler.ReadP2P_Channel_DeathStrandingBackpack();

        Network_Player p = RAPI.GetLocalPlayer();
        if (p)
        {
            Transform bp = p.transform.Find("CharacterPivot").FindChildRecursivly("ChestParent").Find("DeathStrandingBackpack");
            if (bp != null)
            {
                bp.gameObject.SetActive(p.currentModel.thirdPersonSettings.ThirdPersonModel);
            }
        }
    }

    public IEnumerator InventoryUpdater()
    {
        if (!RAPI.IsCurrentSceneMainMenu())
        {
            Network_Player player = RAPI.GetLocalPlayer();
            if (player)
            {
                Dictionary<string, int> currentItems = new Dictionary<string, int>();

                player.Inventory.allSlots.ForEach((slot) =>
                {
                    if (slot.HasValidItemInstance())
                    {
                        if (!currentItems.ContainsKey(slot.itemInstance.baseItem.UniqueName))
                        {
                            currentItems.Add(slot.itemInstance.baseItem.UniqueName, slot.itemInstance.Amount);
                        }
                        else
                        {
                            currentItems[slot.itemInstance.baseItem.UniqueName] += slot.itemInstance.Amount;
                        }
                    }
                });

                if (!currentItems.SequenceEqual(items))
                {
                    items = currentItems;
                    OnInventoryUpdate();
                }
            }
        }
        yield return new WaitForSeconds(0.3f);
        StartCoroutine(InventoryUpdater());
    }

    public void OnInventoryUpdate()
    {
        UpdateUserBackpack(RAPI.GetLocalPlayer(), items);
        MessageHandler.SendMessage(new Message_DeathStrandingBackpackUpdate((Messages)MessageHandler.NetworkMessages.DeathStrandingBackpack_UpdateInventory, RAPI.GetLocalPlayer().steamID, items));
    }

    public override void WorldEvent_WorldLoaded()
    {
        Network_Player player = RAPI.GetLocalPlayer();
        Transform backpackParent = player.transform.Find("CharacterPivot").FindChildRecursivly("ChestParent");
        if (!backpackParent.Find("DeathStrandingBackpack"))
        {
            GameObject backpack = Instantiate(BackpackPrefab, backpackParent.transform.position, backpackParent.transform.rotation, backpackParent);
            backpack.name = "DeathStrandingBackpack";
            UpdateUserBackpack(RAPI.GetLocalPlayer(), items);
        }
    }

    public async void UpdateUserBackpack(Network_Player player, Dictionary<string, int> items)
    {
        GameObject backpack = null;
        Transform backpackParent = player.transform.Find("CharacterPivot").FindChildRecursivly("ChestParent");
        if (!backpackParent.Find("DeathStrandingBackpack"))
        {
            backpack = Instantiate(BackpackPrefab, backpackParent.transform.position, backpackParent.transform.rotation, backpackParent);
            backpack.name = "DeathStrandingBackpack";
            UpdateUserBackpack(player, items);
        }
        else
        {
            backpack = backpackParent.Find("DeathStrandingBackpack").gameObject;
        }

        Debug.Log("Updating backpack of user " + player.characterSettings.Name + " : " + items.Count);

        Transform HoldPoint = backpack.transform.Find("HoldPoint").transform;
        foreach (Transform child in HoldPoint)
        {
            Destroy(child.gameObject);
        }

        float amount = -0.01f;
        float spacing = 0.1f;
        foreach (KeyValuePair<string, int> item in items)
        {
            GameObject GoItem = Instantiate(ItemPrefab, HoldPoint.transform.position + (-HoldPoint.transform.right * amount) + (HoldPoint.transform.up * UnityEngine.Random.Range(-0.03f, 0.03f)) + (HoldPoint.transform.forward * UnityEngine.Random.Range(-0.03f, 0f)), HoldPoint.transform.rotation, HoldPoint);
            GoItem.transform.localRotation = Quaternion.Euler(0, UnityEngine.Random.Range(-2f, 2f), 0);

            Material m = new Material(shader);
            try
            {
                Sprite itemsprite = ItemManager.GetItemByName(item.Key).settings_Inventory.Sprite;
                m.mainTexture = GetTextureFromSprite(itemsprite);
            }
            catch
            {
                m.mainTexture = GetTextureFromSprite(ItemManager.GetItemByIndex(236).settings_Inventory.Sprite);
            }

            m.SetInt("_SrcBlend", (int)UnityEngine.Rendering.BlendMode.One);
            m.SetInt("_DstBlend", (int)UnityEngine.Rendering.BlendMode.OneMinusSrcAlpha);
            m.SetInt("_ZWrite", 0);
            m.DisableKeyword("_ALPHATEST_ON");
            m.DisableKeyword("_ALPHABLEND_ON");
            m.EnableKeyword("_ALPHAPREMULTIPLY_ON");
            m.renderQueue = 1000;

            GoItem.GetComponentsInChildren<Renderer>().ToList().ForEach((r) =>
            {
                if (r.gameObject.name != "model" && r.gameObject.name != "Text" && r.gameObject.name != "StartParticle") { r.material = m; }
            });

            GoItem.GetComponentsInChildren<TextMeshPro>().ToList().ForEach((te) => te.text = "x" + item.Value);

            amount += spacing;
            await Task.Delay(10);
        }
    }

    public Texture2D GetTextureFromSprite(Sprite sprite)
    {
        try
        {
            Texture2D nonReadable = sprite.texture;
            RenderTexture tmp = RenderTexture.GetTemporary(nonReadable.width, nonReadable.height, 0, RenderTextureFormat.Default, RenderTextureReadWrite.sRGB);
            Graphics.Blit(nonReadable, tmp);
            RenderTexture previous = RenderTexture.active;
            RenderTexture.active = tmp;
            Texture2D readable = new Texture2D(nonReadable.width, nonReadable.height);
            readable.ReadPixels(new Rect(0, 0, tmp.width, tmp.height), 0, 0);
            readable.Apply();
            RenderTexture.active = previous;
            RenderTexture.ReleaseTemporary(tmp);
            var croppedTexture = new Texture2D((int)sprite.rect.width, (int)sprite.rect.height);
            var pixels = readable.GetPixels((int)sprite.textureRect.x, (int)sprite.textureRect.y, (int)sprite.textureRect.width, (int)sprite.textureRect.height);
            if (croppedTexture.width != 64)
            {
                return sprite.texture;
            }
            croppedTexture.SetPixels(pixels);
            croppedTexture.Apply();
            return croppedTexture;
        }
        catch
        {
            return GetTextureFromSprite(ItemManager.GetItemByIndex(236).settings_Inventory.Sprite);
        }

    }
}

public class MessageHandler : MonoBehaviour
{
    public enum NetworkMessages
    {
        DeathStrandingBackpack_UpdateInventory = 3249
    }

    public static void SendMessage(Message message)
    {
        if (Semih_Network.IsHost)
        {
            RAPI.GetLocalPlayer().Network.RPC(message, Target.Other, EP2PSend.k_EP2PSendReliable, (NetworkChannel)95);
        }
        else
        {
            RAPI.GetLocalPlayer().SendP2P(message, EP2PSend.k_EP2PSendReliable, (NetworkChannel)95);
        }
    }

    public static void ReadP2P_Channel_DeathStrandingBackpack()
    {
        if (RAPI.IsCurrentSceneMainMenu()) { return; }
        uint num;
        while (SteamNetworking.IsP2PPacketAvailable(out num, 95))
        {
            byte[] array = new byte[num];
            uint num2;
            CSteamID csteamID;
            if (SteamNetworking.ReadP2PPacket(array, num, out num2, out csteamID, 95))
            {
                MemoryStream serializationStream = new MemoryStream(array);
                BinaryFormatter bf = new BinaryFormatter();
                bf.Binder = new PreMergeToMergedDeserializationBinder();
                Packet packet = bf.Deserialize(serializationStream) as Packet;
                Packet_Multiple packet_Multiple;
                if (packet.PacketType == PacketType.Single)
                {
                    Packet_Single packet_Single = packet as Packet_Single;
                    packet_Multiple = new Packet_Multiple(packet_Single.SendType);
                    packet_Multiple.messages = new Message[]
                    {
                        packet_Single.message
                    };
                }
                else
                {
                    packet_Multiple = (packet as Packet_Multiple);
                }
                List<Message> list = packet_Multiple.messages.ToList();
                for (int i = 0; i < list.Count; i++)
                {
                    Message message = list[i];
                    if (message != null)
                    {
                        Messages type = message.Type;
                        switch (type)
                        {
                            case (Messages)NetworkMessages.DeathStrandingBackpack_UpdateInventory:
                                {
                                    Message_DeathStrandingBackpackUpdate msg = message as Message_DeathStrandingBackpackUpdate;
                                    Semih_Network network = ComponentManager<Semih_Network>.Value;
                                    Network_Player player = network.GetPlayerFromID(msg.player);
                                    DeathStrandingBackpack.instance.UpdateUserBackpack(player, msg.items);
                                    break;
                                }
                        }
                    }

                }
            }
        }
    }
}

[Serializable]
public class Message_DeathStrandingBackpackUpdate : Message
{
    public CSteamID player;
    public Dictionary<string, int> items = new Dictionary<string, int>();

    public Message_DeathStrandingBackpackUpdate(Messages type, CSteamID _player, Dictionary<string, int> _items) : base(type)
    {
        player = _player;
        items = _items;
    }
}

sealed class PreMergeToMergedDeserializationBinder : SerializationBinder
{
    public override Type BindToType(string assemblyName, string typeName)
    {
        Type typeToDeserialize = null;
        String exeAssembly = Assembly.GetExecutingAssembly().FullName;
        typeToDeserialize = Type.GetType(String.Format("{0}, {1}", typeName, exeAssembly));
        return typeToDeserialize;
    }
}